@@ -10,6 +10,7 @@
 #include <libtorrent/operations.hpp>
 #include <memory>
 #include "bytes.hpp"
+#include "gil.hpp"
 
 #include <boost/type_traits/is_polymorphic.hpp>
 
@@ -92,7 +93,7 @@ dict dht_immutable_item(dht_immutable_item_alert const& alert)
 {
     dict d;
     d["key"] = alert.target;
-    d["value"] = bytes(alert.item.to_string());
+    d["value"] = bytes(alert.item.string());
     return d;
 }
 
@@ -100,7 +101,7 @@ dict dht_mutable_item(dht_mutable_item_alert const& alert)
 {
     dict d;
     d["key"] = bytes(alert.key.data(), alert.key.size());
-    d["value"] = bytes(alert.item.to_string());
+    d["value"] = bytes(alert.item.string());
     d["signature"] = bytes(alert.signature.data(), alert.signature.size());
     d["seq"] = alert.seq;
     d["salt"] = bytes(alert.salt);
@@ -166,7 +167,8 @@ list dht_sample_infohashes_nodes(dht_sample_infohashes_alert const& alert)
 #if TORRENT_ABI_VERSION == 1
 entry const& get_resume_data_entry(save_resume_data_alert const& self)
 {
-	return *self.resume_data;
+    python_deprecated("resume_data is deprecated");
+    return *self.resume_data;
 }
 #endif
 
@@ -311,7 +313,7 @@ void bind_alert()
             .def("what", &alert::what)
             .def("category", &alert::category)
 #if TORRENT_ABI_VERSION == 1
-            .def("severity", &alert::severity)
+            .def("severity", depr(&alert::severity))
 #endif
             .def("__str__", &alert::message)
             ;
@@ -988,7 +990,7 @@ void bind_alert()
     class_<log_alert, bases<alert>, noncopyable>(
        "log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", &log_alert::msg)
+        .def("msg", depr(&log_alert::msg))
 #endif
         .def("log_message", &log_alert::log_message)
         ;
@@ -996,7 +998,7 @@ void bind_alert()
     class_<torrent_log_alert, bases<torrent_alert>, noncopyable>(
        "torrent_log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", &torrent_log_alert::msg)
+        .def("msg", depr(&torrent_log_alert::msg))
 #endif
         .def("log_message", &torrent_log_alert::log_message)
         ;
@@ -1004,7 +1006,7 @@ void bind_alert()
     class_<peer_log_alert, bases<peer_alert>, noncopyable>(
        "peer_log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", &peer_log_alert::msg)
+        .def("msg", depr(&peer_log_alert::msg))
 #endif
         .def("log_message", &peer_log_alert::log_message)
         ;
@@ -1027,7 +1029,7 @@ void bind_alert()
         ;
 
     class_<dht_log_alert, bases<alert>, noncopyable>("dht_log_alert", no_init)
-        .def_readonly("module", &dht_log_alert::module)
+        .add_property("module", make_getter(&dht_log_alert::module, by_value()))
         .def("log_message", &dht_log_alert::log_message)
     ;
 
@@ -1038,14 +1040,14 @@ void bind_alert()
 
     class_<dht_immutable_item_alert, bases<alert>, noncopyable>(
        "dht_immutable_item_alert", no_init)
-        .def_readonly("target", &dht_immutable_item_alert::target)
+        .add_property("target", make_getter(&dht_immutable_item_alert::target, by_value()))
         .add_property("item", &dht_immutable_item)
         ;
 
     class_<dht_mutable_item_alert, bases<alert>, noncopyable>(
        "dht_mutable_item_alert", no_init)
-        .def_readonly("key", &dht_mutable_item_alert::key)
-        .def_readonly("signature", &dht_mutable_item_alert::signature)
+        .add_property("key", make_getter(&dht_mutable_item_alert::key, by_value()))
+        .add_property("signature", make_getter(&dht_mutable_item_alert::signature, by_value()))
         .def_readonly("seq", &dht_mutable_item_alert::seq)
         .def_readonly("salt", &dht_mutable_item_alert::salt)
         .add_property("item", &dht_mutable_item)
@@ -1054,9 +1056,9 @@ void bind_alert()
 
     class_<dht_put_alert, bases<alert>, noncopyable>(
        "dht_put_alert", no_init)
-        .def_readonly("target", &dht_put_alert::target)
-        .def_readonly("public_key", &dht_put_alert::public_key)
-        .def_readonly("signature", &dht_put_alert::signature)
+        .add_property("target", make_getter(&dht_put_alert::target, by_value()))
+        .add_property("public_key", make_getter(&dht_put_alert::public_key, by_value()))
+        .add_property("signature", make_getter(&dht_put_alert::signature, by_value()))
         .def_readonly("salt", &dht_put_alert::salt)
         .def_readonly("seq", &dht_put_alert::seq)
         .def_readonly("num_success", &dht_put_alert::num_success)
@@ -1083,7 +1085,7 @@ void bind_alert()
     class_<block_uploaded_alert, bases<peer_alert>, noncopyable>(
        "block_uploaded_alert", no_init)
         .add_property("block_index", &block_uploaded_alert::block_index)
-        .add_property("piece_index", make_getter((&block_uploaded_alert::piece_index), by_value()))
+        .add_property("piece_index", make_getter(&block_uploaded_alert::piece_index, by_value()))
         ;
 
     class_<alerts_dropped_alert, bases<alert>, noncopyable>(
