@@ -128,6 +128,8 @@ namespace
 
 			TORRENT_TRY
 			{
+				// if the dictionary doesn't contain "key", it will throw, hence
+				// the try-catch here
 				object const value = sett_dict[key];
 				switch (sett & settings_pack::type_mask)
 				{
@@ -135,8 +137,15 @@ namespace
 						p.set_str(sett, extract<std::string>(value));
 						break;
 					case settings_pack::int_type_base:
-						p.set_int(sett, extract<int>(value));
+					{
+						std::int64_t const val = extract<std::int64_t>(value);
+						// deliberately truncate and sign-convert here. If we
+						// extract an int directly, unsigned ints may throw
+						// an exception otherwise, if it doesn't fit. Notably for a
+						// flag-type with all bits set.
+						p.set_int(sett, static_cast<int>(val));
 						break;
+					}
 					case settings_pack::bool_type_base:
 						p.set_bool(sett, extract<bool>(value));
 						break;
@@ -269,12 +278,14 @@ namespace
 #if TORRENT_ABI_VERSION == 1
             else if(key == "resume_data")
             {
+                python_deprecated("the resume_data member is deprecated");
                 std::string resume = extract<std::string>(value);
                 p.resume_data.assign(resume.begin(), resume.end());
                 continue;
             }
             else if(key == "uuid")
             {
+                python_deprecated("the uuid member is deprecated");
                 p.uuid = extract<std::string>(value);
                 continue;
             }
@@ -329,6 +340,7 @@ namespace
 #if TORRENT_ABI_VERSION == 1
             else if(key == "url")
             {
+                python_deprecated("the url member is deprecated");
                 p.url = extract<std::string>(value);
                 continue;
             }
@@ -362,10 +374,10 @@ namespace
         allow_threading_guard guard;
 
 #ifndef BOOST_NO_EXCEPTIONS
-        return s.add_torrent(p);
+        return s.add_torrent(std::move(p));
 #else
         error_code ec;
-        return s.add_torrent(p, ec);
+        return s.add_torrent(std::move(p), ec);
 #endif
     }
 
@@ -376,7 +388,34 @@ namespace
 
         allow_threading_guard guard;
 
-        s.async_add_torrent(p);
+        s.async_add_torrent(std::move(p));
+    }
+
+    torrent_handle wrap_add_torrent(lt::session& s, lt::add_torrent_params const& p)
+    {
+        add_torrent_params atp = p;
+        if (p.ti)
+            atp.ti = std::make_shared<torrent_info>(*p.ti);
+
+        allow_threading_guard guard;
+
+#ifndef BOOST_NO_EXCEPTIONS
+        return s.add_torrent(std::move(p));
+#else
+        error_code ec;
+        return s.add_torrent(std::move(p), ec);
+#endif
+    }
+
+    void wrap_async_add_torrent(lt::session& s, lt::add_torrent_params const& p)
+    {
+        add_torrent_params atp = p;
+        if (p.ti)
+            atp.ti = std::make_shared<torrent_info>(*p.ti);
+
+        allow_threading_guard guard;
+
+        s.async_add_torrent(std::move(p));
     }
 
 #if TORRENT_ABI_VERSION == 1
@@ -478,8 +517,13 @@ namespace
 
     list get_torrent_status(lt::session& s, object pred, int const flags)
     {
+        // keep a reference to the predicate here, in the python thread, to
+        // ensure it's freed in this thread at the end. If we move it into the
+        // libtorrent thread the python predicate will be freed from that
+        // thread, which won't work
+        auto wrapped_pred = std::bind(&wrap_pred, pred, std::placeholders::_1);
         std::vector<torrent_status> torrents
-            = s.get_torrent_status(std::bind(&wrap_pred, pred, std::placeholders::_1), status_flags_t(flags));
+            = s.get_torrent_status(std::ref(wrapped_pred), status_flags_t(flags));
 
         list ret;
         for (std::vector<torrent_status>::iterator i = torrents.begin(); i != torrents.end(); ++i)
@@ -544,6 +588,7 @@ namespace
 
     dict get_utp_stats(session_status const& st)
     {
+        python_deprecated("session_status is deprecated");
         dict ret;
         ret["num_idle"] = st.utp_stats.num_idle;
         ret["num_syn_sent"] = st.utp_stats.num_syn_sent;
@@ -808,7 +853,7 @@ void bind_session()
         .def_readonly("dht_node_cache", &session_status::dht_node_cache)
         .def_readonly("dht_torrents", &session_status::dht_torrents)
         .def_readonly("dht_global_nodes", &session_status::dht_global_nodes)
-        .def_readonly("active_requests", &session_status::active_requests)
+        .add_property("active_requests", make_getter(&session_status::active_requests, return_value_policy<return_by_value>()))
         .def_readonly("dht_total_allocations", &session_status::dht_total_allocations)
 #endif // TORRENT_DISABLE_DHT
         .add_property("utp_stats", &get_utp_stats)
@@ -1019,6 +1064,7 @@ void bind_session()
 
     {
     scope s = class_<lt::session, boost::noncopyable>("session", no_init)
+        .def(init<>())
         .def("__init__", boost::python::make_constructor(&make_session
                 , default_call_policies()
                 , (arg("settings")
@@ -1031,7 +1077,7 @@ void bind_session()
                 , arg("flags")=lt::session::start_default_features | lt::session::add_default_plugins
                 , arg("alert_mask")=alert::error_notification))
         )
-        .def("outgoing_ports", &outgoing_ports)
+        .def("outgoing_ports", depr(&outgoing_ports))
 #endif
         .def("post_torrent_updates", allow_threads(&lt::session::post_torrent_updates), arg("flags") = 0xffffffff)
         .def("post_dht_stats", allow_threads(&lt::session::post_dht_stats))
@@ -1042,7 +1088,7 @@ void bind_session()
         .def("add_dht_node", &add_dht_node)
 #if TORRENT_ABI_VERSION == 1
         .def(
-            "add_dht_router", &add_dht_router
+            "add_dht_router", depr(&add_dht_router)
           , (arg("router"), "port")
         )
 #endif // TORRENT_ABI_VERSION
@@ -1060,12 +1106,12 @@ void bind_session()
 #endif // TORRENT_DISABLE_DHT
         .def("add_torrent", &add_torrent)
         .def("async_add_torrent", &async_add_torrent)
-        .def("async_add_torrent", static_cast<void (session_handle::*)(lt::add_torrent_params const&)>(&lt::session::async_add_torrent))
-        .def("add_torrent", allow_threads(static_cast<lt::torrent_handle (session_handle::*)(add_torrent_params const&)>(&lt::session::add_torrent)))
+        .def("async_add_torrent", &wrap_async_add_torrent)
+        .def("add_torrent", &wrap_add_torrent)
 #ifndef BOOST_NO_EXCEPTIONS
 #if TORRENT_ABI_VERSION == 1
         .def(
-            "add_torrent", &add_torrent_depr
+            "add_torrent", depr(&add_torrent_depr)
           , (
                 arg("resume_data") = entry(),
                 arg("storage_mode") = storage_mode_sparse,
@@ -1076,14 +1122,14 @@ void bind_session()
 #endif // BOOST_NO_EXCEPTIONS
         .def("remove_torrent", allow_threads(&lt::session::remove_torrent), arg("option") = 0)
 #if TORRENT_ABI_VERSION == 1
-        .def("status", allow_threads(&lt::session::status))
+        .def("status", depr(&lt::session::status))
 #endif
         .def("get_settings", &session_get_settings)
         .def("apply_settings", &session_apply_settings)
 #if TORRENT_ABI_VERSION == 1
 #ifndef TORRENT_DISABLE_ENCRYPTION
-        .def("set_pe_settings", allow_threads(&lt::session::set_pe_settings))
-        .def("get_pe_settings", allow_threads(&lt::session::get_pe_settings))
+        .def("set_pe_settings", depr(&lt::session::set_pe_settings))
+        .def("get_pe_settings", depr(&lt::session::get_pe_settings))
 #endif
 #endif
         .def("load_state", &load_state, (arg("entry"), arg("flags") = 0xffffffff))
@@ -1095,8 +1141,8 @@ void bind_session()
         .def("add_extension", &add_extension)
 #if TORRENT_ABI_VERSION == 1
 #if TORRENT_USE_I2P
-        .def("set_i2p_proxy", allow_threads(&lt::session::set_i2p_proxy))
-        .def("i2p_proxy", allow_threads(&lt::session::i2p_proxy))
+        .def("set_i2p_proxy", depr(&lt::session::set_i2p_proxy))
+        .def("i2p_proxy", depr(&lt::session::i2p_proxy))
 #endif
 #endif
         .def("set_ip_filter", allow_threads(&lt::session::set_ip_filter))
@@ -1120,52 +1166,52 @@ void bind_session()
         .def("set_peer_class", &set_peer_class)
 
 #if TORRENT_ABI_VERSION == 1
-        .def("id", allow_threads(&lt::session::id))
+        .def("id", depr(&lt::session::id))
         .def(
-            "listen_on", &listen_on
+            "listen_on", depr(&listen_on)
           , (arg("min"), "max", arg("interface") = (char const*)nullptr, arg("flags") = 0)
         )
 #ifndef TORRENT_DISABLE_DHT
-        .def("start_dht", allow_threads(start_dht0))
-        .def("stop_dht", allow_threads(&lt::session::stop_dht))
-        .def("start_dht", allow_threads(start_dht1))
-        .def("dht_state", allow_threads(&lt::session::dht_state))
-        .def("set_dht_proxy", allow_threads(&lt::session::set_dht_proxy))
-        .def("dht_proxy", allow_threads(&lt::session::dht_proxy))
+        .def("start_dht", depr(start_dht0))
+        .def("stop_dht", depr(&lt::session::stop_dht))
+        .def("start_dht", depr(start_dht1))
+        .def("dht_state", depr(&lt::session::dht_state))
+        .def("set_dht_proxy", depr(&lt::session::set_dht_proxy))
+        .def("dht_proxy", depr(&lt::session::dht_proxy))
 #endif
-        .def("set_local_download_rate_limit", allow_threads(&lt::session::set_local_download_rate_limit))
-        .def("local_download_rate_limit", allow_threads(&lt::session::local_download_rate_limit))
-        .def("set_local_upload_rate_limit", allow_threads(&lt::session::set_local_upload_rate_limit))
-        .def("local_upload_rate_limit", allow_threads(&lt::session::local_upload_rate_limit))
-        .def("set_download_rate_limit", allow_threads(&lt::session::set_download_rate_limit))
-        .def("download_rate_limit", allow_threads(&lt::session::download_rate_limit))
-        .def("set_upload_rate_limit", allow_threads(&lt::session::set_upload_rate_limit))
-        .def("upload_rate_limit", allow_threads(&lt::session::upload_rate_limit))
-        .def("set_max_uploads", allow_threads(&lt::session::set_max_uploads))
-        .def("set_max_connections", allow_threads(&lt::session::set_max_connections))
-        .def("max_connections", allow_threads(&lt::session::max_connections))
-        .def("num_connections", allow_threads(&lt::session::num_connections))
-        .def("set_max_half_open_connections", allow_threads(&lt::session::set_max_half_open_connections))
-        .def("set_severity_level", allow_threads(&lt::session::set_severity_level))
-        .def("set_alert_queue_size_limit", allow_threads(&lt::session::set_alert_queue_size_limit))
-        .def("set_alert_mask", allow_threads(&lt::session::set_alert_mask))
-        .def("set_peer_proxy", allow_threads(&lt::session::set_peer_proxy))
-        .def("set_tracker_proxy", allow_threads(&lt::session::set_tracker_proxy))
-        .def("set_web_seed_proxy", allow_threads(&lt::session::set_web_seed_proxy))
-        .def("peer_proxy", allow_threads(&lt::session::peer_proxy))
-        .def("tracker_proxy", allow_threads(&lt::session::tracker_proxy))
-        .def("web_seed_proxy", allow_threads(&lt::session::web_seed_proxy))
-        .def("set_proxy", allow_threads(&lt::session::set_proxy))
-        .def("proxy", allow_threads(&lt::session::proxy))
-        .def("start_upnp", &start_upnp)
-        .def("stop_upnp", allow_threads(&lt::session::stop_upnp))
-        .def("start_lsd", allow_threads(&lt::session::start_lsd))
-        .def("stop_lsd", allow_threads(&lt::session::stop_lsd))
-        .def("start_natpmp", &start_natpmp)
-        .def("stop_natpmp", allow_threads(&lt::session::stop_natpmp))
-        .def("get_cache_status", &get_cache_status)
-        .def("get_cache_info", &get_cache_info2)
-        .def("set_peer_id", allow_threads(&lt::session::set_peer_id))
+        .def("set_local_download_rate_limit", depr(&lt::session::set_local_download_rate_limit))
+        .def("local_download_rate_limit", depr(&lt::session::local_download_rate_limit))
+        .def("set_local_upload_rate_limit", depr(&lt::session::set_local_upload_rate_limit))
+        .def("local_upload_rate_limit", depr(&lt::session::local_upload_rate_limit))
+        .def("set_download_rate_limit", depr(&lt::session::set_download_rate_limit))
+        .def("download_rate_limit", depr(&lt::session::download_rate_limit))
+        .def("set_upload_rate_limit", depr(&lt::session::set_upload_rate_limit))
+        .def("upload_rate_limit", depr(&lt::session::upload_rate_limit))
+        .def("set_max_uploads", depr(&lt::session::set_max_uploads))
+        .def("set_max_connections", depr(&lt::session::set_max_connections))
+        .def("max_connections", depr(&lt::session::max_connections))
+        .def("num_connections", depr(&lt::session::num_connections))
+        .def("set_max_half_open_connections", depr(&lt::session::set_max_half_open_connections))
+        .def("set_severity_level", depr(&lt::session::set_severity_level))
+        .def("set_alert_queue_size_limit", depr(&lt::session::set_alert_queue_size_limit))
+        .def("set_alert_mask", depr(&lt::session::set_alert_mask))
+        .def("set_peer_proxy", depr(&lt::session::set_peer_proxy))
+        .def("set_tracker_proxy", depr(&lt::session::set_tracker_proxy))
+        .def("set_web_seed_proxy", depr(&lt::session::set_web_seed_proxy))
+        .def("peer_proxy", depr(&lt::session::peer_proxy))
+        .def("tracker_proxy", depr(&lt::session::tracker_proxy))
+        .def("web_seed_proxy", depr(&lt::session::web_seed_proxy))
+        .def("set_proxy", depr(&lt::session::set_proxy))
+        .def("proxy", depr(&lt::session::proxy))
+        .def("start_upnp", depr(&start_upnp))
+        .def("stop_upnp", depr(&lt::session::stop_upnp))
+        .def("start_lsd", depr(&lt::session::start_lsd))
+        .def("stop_lsd", depr(&lt::session::stop_lsd))
+        .def("start_natpmp", depr(&start_natpmp))
+        .def("stop_natpmp", depr(&lt::session::stop_natpmp))
+        .def("get_cache_status", depr(&get_cache_status))
+        .def("get_cache_info", depr(&get_cache_info2))
+        .def("set_peer_id", depr(&lt::session::set_peer_id))
 #endif // TORRENT_ABI_VERSION
         ;
 
