@@ -31,6 +31,10 @@ settings = {
     'enable_upnp': False, 'listen_interfaces': '0.0.0.0:0', 'file_pool_size': 1}
 
 
+def has_deprecated():
+    return hasattr(lt, 'version')
+
+
 class test_create_torrent(unittest.TestCase):
 
     def test_from_torrent_info(self):
@@ -50,6 +54,8 @@ class test_create_torrent(unittest.TestCase):
         fs = lt.file_storage()
         fs.add_file('test/file1', 1000)
         fs.add_file('test/file2', 2000)
+        self.assertEqual(fs.file_name(0), 'file1')
+        self.assertEqual(fs.file_name(1), 'file2')
         ct = lt.create_torrent(fs)
         ct.add_url_seed('foo')
         ct.add_http_seed('bar')
@@ -57,7 +63,15 @@ class test_create_torrent(unittest.TestCase):
         ct.set_root_cert('1234567890')
         ct.add_collection('1337')
         entry = ct.generate()
-        print(entry)
+        encoded = lt.bencode(entry)
+        print(encoded)
+
+        # zero out the creation date:
+        encoded = encoded.split(b'13:creation datei', 1)
+        encoded[1] = b'0e' + encoded[1].split(b'e', 1)[1]
+        encoded = b'13:creation datei'.join(encoded)
+
+        self.assertEqual(encoded, b'd8:announce3:bar13:creation datei0e9:httpseeds3:bar4:infod11:collectionsl4:1337e5:filesld6:lengthi2000e4:pathl5:file2eed6:lengthi1000e4:pathl5:file1eee4:name4:test12:piece lengthi16384e6:pieces20:\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x008:ssl-cert10:1234567890e8:url-list3:fooe')
 
 
 class test_session_stats(unittest.TestCase):
@@ -69,6 +83,15 @@ class test_session_stats(unittest.TestCase):
             field = getattr(atp, field_name)
             print(field_name, field)
 
+        atp.renamed_files = {}
+        atp.merkle_tree = []
+        atp.unfinished_pieces = {}
+        atp.have_pieces = []
+        atp.banned_peers = []
+        atp.verified_pieces = []
+        atp.piece_priorities = []
+        atp.url_seeds = []
+
     def test_unique(self):
         metrics = lt.session_stats_metrics()
         self.assertTrue(len(metrics) > 40)
@@ -171,6 +194,7 @@ class test_torrent_handle(unittest.TestCase):
             tracker.tier = idx
             tracker.fail_limit = 2
             trackers.append(tracker)
+            self.assertEqual(tracker.url, tracker_url)
         self.h.replace_trackers(trackers)
         new_trackers = self.h.trackers()
         self.assertEqual(new_trackers[0]['url'], 'udp://tracker1.com')
@@ -363,6 +387,8 @@ class TestAddPiece(unittest.TestCase):
 class test_torrent_info(unittest.TestCase):
 
     def test_bencoded_constructor(self):
+        # things that can be converted to a bencoded entry, will be interpreted
+        # as such and encoded
         info = lt.torrent_info({'info': {
             'name': 'test_torrent', 'length': 1234,
             'piece length': 16 * 1024,
@@ -377,6 +403,22 @@ class test_torrent_info(unittest.TestCase):
         self.assertEqual(info.total_size(), 1234)
         self.assertEqual(info.creation_date(), 0)
 
+    def test_bytearray(self):
+        # a bytearray object is interpreted as a bencoded buffer
+        info = lt.torrent_info(bytearray(lt.bencode({'info': {
+            'name': 'test_torrent', 'length': 1234,
+            'piece length': 16 * 1024,
+            'pieces': 'aaaaaaaaaaaaaaaaaaaa'}})))
+        self.assertEqual(info.num_files(), 1)
+
+    def test_bytes(self):
+        # a bytes object is interpreted as a bencoded buffer
+        info = lt.torrent_info(bytes(lt.bencode({'info': {
+            'name': 'test_torrent', 'length': 1234,
+            'piece length': 16 * 1024,
+            'pieces': 'aaaaaaaaaaaaaaaaaaaa'}})))
+        self.assertEqual(info.num_files(), 1)
+
     def test_load_decode_depth_limit(self):
         self.assertRaises(RuntimeError, lambda: lt.torrent_info(
             {'test': {'test': {'test': {'test': {'test': {}}}}}, 'info': {
@@ -404,6 +446,12 @@ class test_torrent_info(unittest.TestCase):
         self.assertTrue(len(ti.metadata()) != 0)
         self.assertTrue(len(ti.hash_for_piece(0)) != 0)
 
+    def test_torrent_info_bytes_overload(self):
+        # bytes will never be interpreted as a file name. It's interpreted as a
+        # bencoded buffer
+        with self.assertRaises(RuntimeError):
+            ti = lt.torrent_info(b'base.torrent')
+
     def test_web_seeds(self):
         ti = lt.torrent_info('base.torrent')
 
@@ -424,6 +472,41 @@ class test_torrent_info(unittest.TestCase):
         self.assertEqual(ae.verified, False)
         self.assertEqual(ae.source, 0)
 
+    def test_torrent_info_sha1_hash_overload(self):
+        ti = lt.torrent_info(lt.sha1_hash('a' * 20))
+        self.assertEqual(ti.info_hash(), lt.sha1_hash('a' * 20))
+
+        ti_copy = lt.torrent_info(ti)
+        self.assertEqual(ti_copy.info_hash(), lt.sha1_hash('a' * 20))
+
+    def test_url_seed(self):
+        ti = lt.torrent_info('base.torrent')
+
+        ti.add_tracker('foobar1')
+        ti.add_url_seed('foobar2')
+        ti.add_url_seed('foobar3', 'username:password')
+        ti.add_url_seed('foobar4', 'username:password', [])
+
+        seeds = ti.web_seeds()
+        self.assertEqual(seeds, [
+            {'url': 'foobar2', 'type': 0, 'auth': ''},
+            {'url': 'foobar3', 'type': 0, 'auth': 'username:password'},
+            {'url': 'foobar4', 'type': 0, 'auth': 'username:password'},
+        ])
+
+    def test_http_seed(self):
+        ti = lt.torrent_info('base.torrent')
+
+        ti.add_http_seed('foobar2')
+        ti.add_http_seed('foobar3', 'username:password')
+        ti.add_http_seed('foobar4', 'username:password', [])
+
+        seeds = ti.web_seeds()
+        self.assertEqual(seeds, [
+            {'url': 'foobar2', 'type': 1, 'auth': ''},
+            {'url': 'foobar3', 'type': 1, 'auth': 'username:password'},
+            {'url': 'foobar4', 'type': 1, 'auth': 'username:password'},
+        ])
 
 class test_alerts(unittest.TestCase):
 
@@ -518,16 +601,40 @@ class test_alerts(unittest.TestCase):
 class test_bencoder(unittest.TestCase):
 
     def test_bencode(self):
-
         encoded = lt.bencode({'a': 1, 'b': [1, 2, 3], 'c': 'foo'})
         self.assertEqual(encoded, b'd1:ai1e1:bli1ei2ei3ee1:c3:fooe')
 
     def test_bdecode(self):
-
         encoded = b'd1:ai1e1:bli1ei2ei3ee1:c3:fooe'
         decoded = lt.bdecode(encoded)
         self.assertEqual(decoded, {b'a': 1, b'b': [1, 2, 3], b'c': b'foo'})
 
+    def test_string(self):
+        encoded = lt.bencode('foo\u00e5\u00e4\u00f6')
+        self.assertEqual(encoded, b'9:foo\xc3\xa5\xc3\xa4\xc3\xb6')
+
+    def test_bytes(self):
+        encoded = lt.bencode(b'foo')
+        self.assertEqual(encoded, b'3:foo')
+
+    def test_float(self):
+        # TODO: this should throw a TypeError in the future
+        with self.assertWarns(DeprecationWarning):
+            encoded = lt.bencode(1.337)
+            self.assertEqual(encoded, b'0:')
+
+    def test_object(self):
+        class FooBar:
+            dummy = 1
+
+        # TODO: this should throw a TypeError in the future
+        with self.assertWarns(DeprecationWarning):
+            encoded = lt.bencode(FooBar())
+            self.assertEqual(encoded, b'0:')
+
+    def test_preformatted(self):
+        encoded = lt.bencode((1, 2, 3, 4, 5))
+        self.assertEqual(encoded, b'\x01\x02\x03\x04\x05')
 
 class test_sha1hash(unittest.TestCase):
 
@@ -611,9 +718,28 @@ class test_peer_class(unittest.TestCase):
         s.set_peer_class_type_filter(lt.peer_class_type_filter())
         s.set_peer_class_filter(lt.ip_filter())
 
+class test_ip_filter(unittest.TestCase):
+
+    def test_export(self):
+
+        f = lt.ip_filter()
+        self.assertEqual(f.access('1.1.1.1'), 0)
+        f.add_rule('1.1.1.1', '1.1.1.2', 1)
+        self.assertEqual(f.access('1.1.1.0'), 0)
+        self.assertEqual(f.access('1.1.1.1'), 1)
+        self.assertEqual(f.access('1.1.1.2'), 1)
+        self.assertEqual(f.access('1.1.1.3'), 0)
+        exp = f.export_filter()
+        self.assertEqual(exp, ([('0.0.0.0', '1.1.1.0'), ('1.1.1.1', '1.1.1.2'), ('1.1.1.3', '255.255.255.255')], [('::', 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')]))
 
 class test_session(unittest.TestCase):
 
+    def test_settings(self):
+        sett = { 'alert_mask': lt.alert.category_t.all_categories }
+        s = lt.session(sett)
+        sett = s.get_settings()
+        self.assertEqual(sett['alert_mask'] & 0x7fffffff, 0x7fffffff)
+
     def test_add_torrent(self):
         s = lt.session(settings)
         s.add_torrent({'ti': lt.torrent_info('base.torrent'),
@@ -624,6 +750,20 @@ class test_session(unittest.TestCase):
                        'banned_peers': [('8.7.6.5', 6881)],
                        'file_priorities': [1, 1, 1, 2, 0]})
 
+    def test_session_status(self):
+        if not has_deprecated():
+            return
+
+        s = lt.session()
+        st = s.status()
+        print(st)
+        print(st.active_requests)
+        print(st.dht_nodes)
+        print(st.dht_node_cache)
+        print(st.dht_torrents)
+        print(st.dht_global_nodes)
+        print(st.dht_total_allocations)
+
     def test_apply_settings(self):
 
         s = lt.session(settings)
@@ -856,10 +996,19 @@ class test_dht_settings(unittest.TestCase):
 
 if __name__ == '__main__':
     print(lt.__version__)
-    shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
-                             'url_seed_multi.torrent'), '.')
-    shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
-                             'base.torrent'), '.')
-    shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
-                             'unordered.torrent'), '.')
+    try:
+        shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
+                                 'url_seed_multi.torrent'), '.')
+    except shutil.SameFileError:
+        pass
+    try:
+        shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
+                                 'base.torrent'), '.')
+    except shutil.SameFileError:
+        pass
+    try:
+        shutil.copy(os.path.join('..', '..', 'test', 'test_torrents',
+                                 'unordered.torrent'), '.')
+    except shutil.SameFileError:
+        pass
     unittest.main()
